# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2024/02/14 17:48
describe: 时序特征因子库

因子函数编写规范：https://s0cqcxuy3p.feishu.cn/wiki/A9yawT6o1il9SrkUoBNchtXjnBK
"""

from .ret import (
    RET001,
    RET002,
    RET003,
    RET004,
    RET005,
    RET006,
    RET007,
    RET008,
)

from .vpf import (
    VPF001,
    VPF002,
    VPF003,
    VPF004,
)

from .tas import (
    CCF
)